package at.ac.uibk.gitsearch.service;

import at.ac.uibk.gitsearch.repository.StatisticsRepository;
import at.ac.uibk.gitsearch.service.dto.StatisticsDTO;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import org.codeability.sharing.plugins.api.search.SearchResultDTO;
import org.codeability.sharing.plugins.api.search.SearchResultsDTO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional
public class AchievementService {

    private final Logger log = LoggerFactory.getLogger(AchievementService.class);

    private final SearchService searchService;

    private final StatisticsRepository statisticsRepository;

    public AchievementService(SearchService searchService, StatisticsRepository statisticsRepository) {
        this.searchService = searchService;
        this.statisticsRepository = statisticsRepository;
    }

    public StatisticsDTO searchUserStatistics(String email) throws IOException {
        SearchResultsDTO results = searchService.searchResultForAuthorEmail(email, 1000);

        List<SearchResultDTO> result = results.getSearchResult();
        List<String> query = new ArrayList<>();
        for (SearchResultDTO res: result) {
            query.add(res.getExerciseId());
        }

        StatisticsDTO toReturn = new StatisticsDTO();

        List<Integer[]> data = statisticsRepository.findNumberOfDownloadsAndViewsByExerciseIDs(query);

        if (data.get(0)[0] != null) {
            toReturn.setDownloads(data.get(0)[0]);
        } else {
            toReturn.setDownloads(0);
        }
        if (data.get(0)[1] != null) {
            toReturn.setViews(data.get(0)[1]);
        } else {
            toReturn.setViews(0);
        }

        log.info("Returning " + data.get(0)[1] + " views and " + data.get(0)[0] + " downloads");
        return toReturn;
    }
}
// Maybe to use later
// Optional<StatisticsDTO> statisticsDTO = statisticsService.findOneByExerciseID((result.getExerciseId()));
// if (statisticsDTO.isPresent()) {
// StatisticsDTO newStats = statisticsDTO.get();
//     totalAmountOfViews += newStats.getViews();
//     totalAmountOfDownloads += newStats.getDownloads();
// }
